@@ -20,6 +20,7 @@ build a Genetic Program Tree, and the functions to evaluate it.
 This module support both strongly and loosely typed GP.
 """
 import copy
+import math
 import random
 import re
 import sys
@@ -30,6 +31,8 @@ from functools import partial, wraps
 from inspect import isclass
 from operator import eq, lt
 
+import tools  # Needed by HARM-GP
+
 ######################################
 # GP Data structure                  #
 ######################################
@@ -37,14 +40,16 @@ from operator import eq, lt
 # Define the name of type for any types.
 __type__ = object
 
+
 class PrimitiveTree(list):
-    """Tree spefically formated for optimization of genetic
+    """Tree specifically formatted for optimization of genetic
     programming operations. The tree is represented with a
     list where the nodes are appended in a depth-first order.
     The nodes appended to the tree are required to
     have an attribute *arity* which defines the arity of the
     primitive. An arity of 0 is expected from terminals nodes.
     """
+
     def __init__(self, content):
         list.__init__(self, content)
 
@@ -59,22 +64,22 @@ class PrimitiveTree(list):
         if isinstance(key, slice):
             if key.start >= len(self):
                 raise IndexError("Invalid slice object (try to assign a %s"
-                    " in a tree of size %d). Even if this is allowed by the"
-                    " list object slice setter, this should not be done in"
-                    " the PrimitiveTree context, as this may lead to an"
-                    " unpredictable behavior for searchSubtree or evaluate."
-                     % (key, len(self)))
+                                 " in a tree of size %d). Even if this is allowed by the"
+                                 " list object slice setter, this should not be done in"
+                                 " the PrimitiveTree context, as this may lead to an"
+                                 " unpredictable behavior for searchSubtree or evaluate."
+                                 % (key, len(self)))
             total = val[0].arity
             for node in val[1:]:
                 total += node.arity - 1
             if total != 0:
                 raise ValueError("Invalid slice assignation : insertion of"
-                    " an incomplete subtree is not allowed in PrimitiveTree."
-                    " A tree is defined as incomplete when some nodes cannot"
-                    " be mapped to any position in the tree, considering the"
-                    " primitives' arity. For instance, the tree [sub, 4, 5,"
-                    " 6] is incomplete if the arity of sub is 2, because it"
-                    " would produce an orphan node (the 6).")
+                                 " an incomplete subtree is not allowed in PrimitiveTree."
+                                 " A tree is defined as incomplete when some nodes cannot"
+                                 " be mapped to any position in the tree, considering the"
+                                 " primitives' arity. For instance, the tree [sub, 4, 5,"
+                                 " 6] is incomplete if the arity of sub is 2, because it"
+                                 " would produce an orphan node (the 6).")
         elif val.arity != self[key].arity:
             raise ValueError("Invalid node replacement with a node of a"
                              " different arity.")
@@ -91,7 +96,7 @@ class PrimitiveTree(list):
                 prim, args = stack.pop()
                 string = prim.format(*args)
                 if len(stack) == 0:
-                    break   # If stack is empty, all nodes should have been seen
+                    break  # If stack is empty, all nodes should have been seen
                 stack[-1][1].append(string)
 
         return string
@@ -155,7 +160,7 @@ class PrimitiveTree(list):
         for elem in self:
             depth = stack.pop()
             max_depth = max(max_depth, depth)
-            stack.extend([depth+1] * elem.arity)
+            stack.extend([depth + 1] * elem.arity)
         return max_depth
 
     @property
@@ -186,6 +191,7 @@ class Primitive(object):
         'mul(1, 2)'
     """
     __slots__ = ('name', 'arity', 'args', 'ret', 'seq')
+
     def __init__(self, name, args, ret):
         self.name = name
         self.arity = len(args)
@@ -197,11 +203,20 @@ class Primitive(object):
     def format(self, *args):
         return self.seq.format(*args)
 
+    def __eq__(self, other):
+        if type(self) is type(other):
+            return all(getattr(self, slot) == getattr(other, slot)
+                       for slot in self.__slots__)
+        else:
+            return NotImplemented
+
+
 class Terminal(object):
     """Class that encapsulates terminal primitive in expression. Terminals can
     be values or 0-arity functions.
     """
     __slots__ = ('name', 'value', 'ret', 'conv_fct')
+
     def __init__(self, terminal, symbolic, ret):
         self.ret = ret
         self.value = terminal
@@ -215,12 +230,21 @@ class Terminal(object):
     def format(self):
         return self.conv_fct(self.value)
 
+    def __eq__(self, other):
+        if type(self) is type(other):
+            return all(getattr(self, slot) == getattr(other, slot)
+                       for slot in self.__slots__)
+        else:
+            return NotImplemented
+
+
 class Ephemeral(Terminal):
     """Class that encapsulates a terminal which value is set when the
     object is created. To mutate the value, a new object has to be
     generated. This is an abstract base class. When subclassing, a
     staticmethod 'func' must be defined.
     """
+
     def __init__(self):
         Terminal.__init__(self, self.func(), symbolic=False, ret=self.ret)
 
@@ -230,11 +254,13 @@ class Ephemeral(Terminal):
         """
         raise NotImplementedError
 
+
 class PrimitiveSetTyped(object):
     """Class that contains the primitives that can be used to solve a
     Strongly Typed GP problem. The set also defined the researched
     function return type, and input arguments type and number.
     """
+
     def __init__(self, name, in_types, ret_type, prefix="ARG"):
         self.terminals = defaultdict(list)
         self.primitives = defaultdict(list)
@@ -242,7 +268,7 @@ class PrimitiveSetTyped(object):
         # setting "__builtins__" to None avoid the context
         # being polluted by builtins function when evaluating
         # GP expression.
-        self.context = {"__builtins__" : None}
+        self.context = {"__builtins__": None}
         self.mapping = dict()
         self.terms_count = 0
         self.prims_count = 0
@@ -270,12 +296,12 @@ class PrimitiveSetTyped(object):
 
     def _add(self, prim):
         def addType(dict_, ret_type):
-            if not ret_type in dict_:
+            if ret_type not in dict_:
                 new_list = []
                 for type_, list_ in dict_.items():
                     if issubclass(type_, ret_type):
                         for item in list_:
-                            if not item in new_list:
+                            if item not in new_list:
                                 new_list.append(item)
                 dict_[ret_type] = new_list
 
@@ -310,9 +336,9 @@ class PrimitiveSetTyped(object):
 
         assert name not in self.context or \
                self.context[name] is primitive, \
-               "Primitives are required to have a unique name. " \
-               "Consider using the argument 'name' to rename your "\
-               "second '%s' primitive." % (name,)
+            "Primitives are required to have a unique name. " \
+            "Consider using the argument 'name' to rename your " \
+            "second '%s' primitive." % (name,)
 
         self._add(prim)
         self.context[prim.name] = primitive
@@ -336,9 +362,9 @@ class PrimitiveSetTyped(object):
             name = terminal.__name__
 
         assert name not in self.context, \
-               "Terminals are required to have a unique name. " \
-               "Consider using the argument 'name' to rename your "\
-               "second %s terminal." % (name,)
+            "Terminals are required to have a unique name. " \
+            "Consider using the argument 'name' to rename your " \
+            "second %s terminal." % (name,)
 
         if name is not None:
             self.context[name] = terminal
@@ -363,9 +389,9 @@ class PrimitiveSetTyped(object):
         :param ret_type: type of the object returned by *ephemeral*.
         """
         module_gp = globals()
-        if not name in module_gp:
-            class_ = type(name, (Ephemeral,), {'func' : staticmethod(ephemeral),
-                                               'ret' : ret_type})
+        if name not in module_gp:
+            class_ = type(name, (Ephemeral,), {'func': staticmethod(ephemeral),
+                                               'ret': ret_type})
             module_gp[name] = class_
         else:
             class_ = module_gp[name]
@@ -379,7 +405,7 @@ class PrimitiveSetTyped(object):
             else:
                 raise Exception("Ephemerals should be named differently "
                                 "than classes defined in the gp module.")
-        
+
         self._add(class_)
         self.terms_count += 1
 
@@ -400,12 +426,14 @@ class PrimitiveSetTyped(object):
         """
         return self.terms_count / float(self.terms_count + self.prims_count)
 
+
 class PrimitiveSet(PrimitiveSetTyped):
     """Class same as :class:`~deap.gp.PrimitiveSetTyped`, except there is no
     definition of type.
     """
+
     def __init__(self, name, arity, prefix="ARG"):
-        args = [__type__]*arity
+        args = [__type__] * arity
         PrimitiveSetTyped.__init__(self, name, args, __type__, prefix)
 
     def addPrimitive(self, primitive, arity, name=None):
@@ -451,10 +479,11 @@ def compile(expr, pset):
     except MemoryError:
         _, _, traceback = sys.exc_info()
         raise MemoryError, ("DEAP : Error in tree evaluation :"
-        " Python cannot evaluate a tree higher than 90. "
-        "To avoid this problem, you should use bloat control on your "
-        "operators. See the DEAP documentation for more information. "
-        "DEAP will now abort."), traceback
+                            " Python cannot evaluate a tree higher than 90. "
+                            "To avoid this problem, you should use bloat control on your "
+                            "operators. See the DEAP documentation for more information. "
+                            "DEAP will now abort."), traceback
+
 
 def compileADF(expr, psets):
     """Compile the expression represented by a list of trees. The first
@@ -478,9 +507,10 @@ def compileADF(expr, psets):
     for pset, subexpr in reversed(zip(psets, expr)):
         pset.context.update(adfdict)
         func = compile(subexpr, pset)
-        adfdict.update({pset.name : func})
+        adfdict.update({pset.name: func})
     return func
 
+
 ######################################
 # GP Program generation functions    #
 ######################################
@@ -492,14 +522,18 @@ def genFull(pset, min_, max_, type_=None):
     :param min_: Minimum height of the produced trees.
     :param max_: Maximum Height of the produced trees.
     :param type_: The type that should return the tree when called, when
-                  :obj:`None` (default) no return type is enforced.
+                  :obj:`None` (default) the type of :pset: (pset.ret)
+                  is assumed.
     :returns: A full tree with all leaves at the same depth.
     """
+
     def condition(height, depth):
         """Expression generation stops when the depth is equal to height."""
         return depth == height
+
     return generate(pset, min_, max_, condition, type_)
 
+
 def genGrow(pset, min_, max_, type_=None):
     """Generate an expression where each leaf might have a different depth
     between *min* and *max*.
@@ -508,17 +542,21 @@ def genGrow(pset, min_, max_, type_=None):
     :param min_: Minimum height of the produced trees.
     :param max_: Maximum Height of the produced trees.
     :param type_: The type that should return the tree when called, when
-                  :obj:`None` (default) no return type is enforced.
+                  :obj:`None` (default) the type of :pset: (pset.ret)
+                  is assumed.
     :returns: A grown tree with leaves at possibly different depths.
     """
+
     def condition(height, depth):
         """Expression generation stops when the depth is equal to height
         or when it is randomly determined that a a node should be a terminal.
         """
         return depth == height or \
                (depth >= min_ and random.random() < pset.terminalRatio)
+
     return generate(pset, min_, max_, condition, type_)
 
+
 def genHalfAndHalf(pset, min_, max_, type_=None):
     """Generate an expression with a PrimitiveSet *pset*.
     Half the time, the expression is generated with :func:`~deap.gp.genGrow`,
@@ -528,12 +566,14 @@ def genHalfAndHalf(pset, min_, max_, type_=None):
     :param min_: Minimum height of the produced trees.
     :param max_: Maximum Height of the produced trees.
     :param type_: The type that should return the tree when called, when
-                  :obj:`None` (default) no return type is enforced.
+                  :obj:`None` (default) the type of :pset: (pset.ret)
+                  is assumed.
     :returns: Either, a full or a grown tree.
     """
     method = random.choice((genGrow, genFull))
     return method(pset, min_, max_, type_)
 
+
 def genRamped(pset, min_, max_, type_=None):
     """
     .. deprecated:: 1.0
@@ -543,6 +583,7 @@ def genRamped(pset, min_, max_, type_=None):
                   FutureWarning)
     return genHalfAndHalf(pset, min_, max_, type_)
 
+
 def generate(pset, min_, max_, condition, type_=None):
     """Generate a Tree as a list of list. The tree is build
     from the root to the leaves, and it stop growing when the
@@ -555,7 +596,8 @@ def generate(pset, min_, max_, condition, type_=None):
                       the height of the tree to build and the current
                       depth in the tree.
     :param type_: The type that should return the tree when called, when
-                  :obj:`None` (default) no return type is enforced.
+                  :obj:`None` (default) the type of :pset: (pset.ret)
+                  is assumed.
     :returns: A grown tree with leaves at possibly different depths
               dependending on the condition function.
     """
@@ -571,8 +613,8 @@ def generate(pset, min_, max_, condition, type_=None):
                 term = random.choice(pset.terminals[type_])
             except IndexError:
                 _, _, traceback = sys.exc_info()
-                raise IndexError, "The gp.generate function tried to add "\
-                                  "a terminal of type '%s', but there is "\
+                raise IndexError, "The gp.generate function tried to add " \
+                                  "a terminal of type '%s', but there is " \
                                   "none available." % (type_,), traceback
             if isclass(term):
                 term = term()
@@ -582,12 +624,12 @@ def generate(pset, min_, max_, condition, type_=None):
                 prim = random.choice(pset.primitives[type_])
             except IndexError:
                 _, _, traceback = sys.exc_info()
-                raise IndexError, "The gp.generate function tried to add "\
-                                  "a primitive of type '%s', but there is "\
+                raise IndexError, "The gp.generate function tried to add " \
+                                  "a primitive of type '%s', but there is " \
                                   "none available." % (type_,), traceback
             expr.append(prim)
             for arg in reversed(prim.args):
-                stack.append((depth+1, arg))
+                stack.append((depth + 1, arg))
     return expr
 
 
@@ -724,17 +766,18 @@ def mutNodeReplacement(individual, pset):
     index = random.randrange(1, len(individual))
     node = individual[index]
 
-    if node.arity == 0: # Terminal
+    if node.arity == 0:  # Terminal
         term = random.choice(pset.terminals[node.ret])
         if isclass(term):
             term = term()
         individual[index] = term
-    else:   # Primitive
+    else:  # Primitive
         prims = [p for p in pset.primitives[node.ret] if p.args == node.args]
         individual[index] = random.choice(prims)
 
     return individual,
 
+
 def mutEphemeral(individual, mode):
     """This operator works on the constants of the tree *individual*. In
     *mode* ``"one"``, it will change the value of one of the individual
@@ -762,6 +805,7 @@ def mutEphemeral(individual, mode):
 
     return individual,
 
+
 def mutInsert(individual, pset):
     """Inserts a new branch at a random position in *individual*. The subtree
     at the chosen position is used as child node of the created subtree, in
@@ -796,11 +840,12 @@ def mutInsert(individual, pset):
                 term = term()
             new_subtree[i] = term
 
-    new_subtree[position:position+1] = individual[slice_]
+    new_subtree[position:position + 1] = individual[slice_]
     new_subtree.insert(0, new_node)
     individual[slice_] = new_subtree
     return individual,
 
+
 def mutShrink(individual):
     """This operator shrinks the *individual* by chosing randomly a branch and
     replacing it with one of the branch's arguments (also randomly chosen).
@@ -820,8 +865,8 @@ def mutShrink(individual):
     if len(iprims) != 0:
         index, prim = random.choice(iprims)
         arg_idx = random.choice([i for i, type_ in enumerate(prim.args) if type_ == prim.ret])
-        rindex = index+1
-        for _ in range(arg_idx+1):
+        rindex = index + 1
+        for _ in range(arg_idx + 1):
             rslice = individual.searchSubtree(rindex)
             subtree = individual[rslice]
             rindex += len(subtree)
@@ -831,10 +876,12 @@ def mutShrink(individual):
 
     return individual,
 
+
 ######################################
 # GP bloat control decorators        #
 ######################################
 
+
 def staticLimit(key, max_value):
     """Implement a static limit on some measurement on a GP tree, as defined
     by Koza in [Koza1989]. It may be used to decorate both crossover and
@@ -846,8 +893,8 @@ def staticLimit(key, max_value):
     depth), because it can ensure that no tree higher than this limit will ever
     be accepted in the population, except if it was generated at initialization
     time.
-    
-    :param key: The function to use in order the get the wanted value. For 
+
+    :param key: The function to use in order the get the wanted value. For
                 instance, on a GP tree, ``operator.attrgetter('height')`` may
                 be used to set a depth limit, and ``len`` to set a size limit.
     :param max_value: The maximum value allowed for the given measurement.
@@ -863,6 +910,7 @@ def staticLimit(key, max_value):
         Cambridge, MA, 1992)
 
     """
+
     def decorator(func):
         @wraps(func)
         def wrapper(*args, **kwargs):
@@ -872,9 +920,216 @@ def staticLimit(key, max_value):
                 if key(ind) > max_value:
                     new_inds[i] = random.choice(keep_inds)
             return new_inds
+
         return wrapper
+
     return decorator
 
+
+######################################
+# GP bloat control algorithms        #
+######################################
+
+def harm(population, toolbox, cxpb, mutpb, ngen,
+         alpha, beta, gamma, rho, nbrindsmodel=-1, mincutoff=20,
+         stats=None, halloffame=None, verbose=__debug__):
+    """Implement bloat control on a GP evolution using HARM-GP, as defined in
+    [Gardner2015]. It is implemented in the form of an evolution algorithm
+    (similar to :func:`~deap.algorithms.eaSimple`).
+
+    :param population: A list of individuals.
+    :param toolbox: A :class:`~deap.base.Toolbox` that contains the evolution
+                    operators.
+    :param cxpb: The probability of mating two individuals.
+    :param mutpb: The probability of mutating an individual.
+    :param ngen: The number of generation.
+    :param alpha: The HARM *alpha* parameter.
+    :param beta: The HARM *beta* parameter.
+    :param gamma: The HARM *gamma* parameter.
+    :param rho: The HARM *rho* parameter.
+    :param nbrindsmodel: The number of individuals to generate in order to
+                            model the natural distribution. -1 is a special
+                            value which uses the equation proposed in
+                            [Gardner2015] to set the value of this parameter :
+                            max(2000, len(population))
+    :param mincutoff: The absolute minimum value for the cutoff point. It is
+                        used to ensure that HARM does not shrink the population
+                        too much at the beginning of the evolution. The default
+                        value is usually fine.
+    :param stats: A :class:`~deap.tools.Statistics` object that is updated
+                  inplace, optional.
+    :param halloffame: A :class:`~deap.tools.HallOfFame` object that will
+                       contain the best individuals, optional.
+    :param verbose: Whether or not to log the statistics.
+    :returns: The final population
+    :returns: A class:`~deap.tools.Logbook` with the statistics of the
+              evolution
+
+    This function expects the :meth:`toolbox.mate`, :meth:`toolbox.mutate`,
+    :meth:`toolbox.select` and :meth:`toolbox.evaluate` aliases to be
+    registered in the toolbox.
+
+    .. note::
+       The recommended values for the HARM-GP parameters are *alpha=0.05*,
+       *beta=10*, *gamma=0.25*, *rho=0.9*. However, these parameters can be
+       adjusted to perform better on a specific problem (see the relevant
+       paper for tuning information). The number of individuals used to
+       model the natural distribution and the minimum cutoff point are less
+       important, their default value being effective in most cases.
+
+    .. [Gardner2015] M.-A. Gardner, C. Gagne, and M. Parizeau, Controlling
+        Code Growth by Dynamically Shaping the Genotype Size Distribution,
+        Genetic Programming and Evolvable Machines, 2015,
+        DOI 10.1007/s10710-015-9242-8
+
+    """
+
+    def _genpop(n, pickfrom=[], acceptfunc=lambda s: True, producesizes=False):
+        # Generate a population of n individuals, using individuals in
+        # *pickfrom* if possible, with a *acceptfunc* acceptance function.
+        # If *producesizes* is true, also return a list of the produced
+        # individuals sizes.
+        # This function is used 1) to generate the natural distribution
+        # (in this case, pickfrom and acceptfunc should be let at their
+        # default values) and 2) to generate the final population, in which
+        # case pickfrom should be the natural population previously generated
+        # and acceptfunc a function implementing the HARM-GP algorithm.
+        producedpop = []
+        producedpopsizes = []
+        while len(producedpop) < n:
+            if len(pickfrom) > 0:
+                # If possible, use the already generated
+                # individuals (more efficient)
+                aspirant = pickfrom.pop()
+                if acceptfunc(len(aspirant)):
+                    producedpop.append(aspirant)
+                    if producesizes:
+                        producedpopsizes.append(len(aspirant))
+            else:
+                opRandom = random.random()
+                if opRandom < cxpb:
+                    # Crossover
+                    aspirant1, aspirant2 = toolbox.mate(*map(toolbox.clone,
+                                                             toolbox.select(population, 2)))
+                    del aspirant1.fitness.values, aspirant2.fitness.values
+                    if acceptfunc(len(aspirant1)):
+                        producedpop.append(aspirant1)
+                        if producesizes:
+                            producedpopsizes.append(len(aspirant1))
+
+                    if len(producedpop) < n and acceptfunc(len(aspirant2)):
+                        producedpop.append(aspirant2)
+                        if producesizes:
+                            producedpopsizes.append(len(aspirant2))
+                else:
+                    aspirant = toolbox.clone(toolbox.select(population, 1)[0])
+                    if opRandom - cxpb < mutpb:
+                        # Mutation
+                        aspirant = toolbox.mutate(aspirant)[0]
+                        del aspirant.fitness.values
+                    if acceptfunc(len(aspirant)):
+                        producedpop.append(aspirant)
+                        if producesizes:
+                            producedpopsizes.append(len(aspirant))
+
+        if producesizes:
+            return producedpop, producedpopsizes
+        else:
+            return producedpop
+
+    def halflifefunc(x):
+        return x * float(alpha) + beta
+
+    if nbrindsmodel == -1:
+        nbrindsmodel = max(2000, len(population))
+
+    logbook = tools.Logbook()
+    logbook.header = ['gen', 'nevals'] + (stats.fields if stats else [])
+
+    # Evaluate the individuals with an invalid fitness
+    invalid_ind = [ind for ind in population if not ind.fitness.valid]
+    fitnesses = toolbox.map(toolbox.evaluate, invalid_ind)
+    for ind, fit in zip(invalid_ind, fitnesses):
+        ind.fitness.values = fit
+
+    if halloffame is not None:
+        halloffame.update(population)
+
+    record = stats.compile(population) if stats else {}
+    logbook.record(gen=0, nevals=len(invalid_ind), **record)
+    if verbose:
+        print logbook.stream
+
+    # Begin the generational process
+    for gen in range(1, ngen + 1):
+        # Estimation population natural distribution of sizes
+        naturalpop, naturalpopsizes = _genpop(nbrindsmodel, producesizes=True)
+
+        naturalhist = [0] * (max(naturalpopsizes) + 3)
+        for indsize in naturalpopsizes:
+            # Kernel density estimation application
+            naturalhist[indsize] += 0.4
+            naturalhist[indsize - 1] += 0.2
+            naturalhist[indsize + 1] += 0.2
+            naturalhist[indsize + 2] += 0.1
+            if indsize - 2 >= 0:
+                naturalhist[indsize - 2] += 0.1
+
+        # Normalization
+        naturalhist = [val * len(population) / nbrindsmodel for val in naturalhist]
+
+        # Cutoff point selection
+        sortednatural = sorted(naturalpop, key=lambda ind: ind.fitness)
+        cutoffcandidates = sortednatural[int(len(population) * rho - 1):]
+        # Select the cutoff point, with an absolute minimum applied
+        # to avoid weird cases in the first generations
+        cutoffsize = max(mincutoff, len(min(cutoffcandidates, key=len)))
+
+        # Compute the target distribution
+        def targetfunc(x):
+            return (gamma * len(population) * math.log(2) /
+                    halflifefunc(x)) * math.exp(-math.log(2) *
+                                                (x - cutoffsize) / halflifefunc(x))
+
+        targethist = [naturalhist[binidx] if binidx <= cutoffsize else
+                      targetfunc(binidx) for binidx in range(len(naturalhist))]
+
+        # Compute the probabilities distribution
+        probhist = [t / n if n > 0 else t for n, t in zip(naturalhist, targethist)]
+
+        def probfunc(s):
+            return probhist[s] if s < len(probhist) else targetfunc(s)
+
+        def acceptfunc(s):
+            return random.random() <= probfunc(s)
+
+        # Generate offspring using the acceptance probabilities
+        # previously computed
+        offspring = _genpop(len(population), pickfrom=naturalpop,
+                            acceptfunc=acceptfunc, producesizes=False)
+
+        # Evaluate the individuals with an invalid fitness
+        invalid_ind = [ind for ind in offspring if not ind.fitness.valid]
+        fitnesses = toolbox.map(toolbox.evaluate, invalid_ind)
+        for ind, fit in zip(invalid_ind, fitnesses):
+            ind.fitness.values = fit
+
+        # Update the hall of fame with the generated individuals
+        if halloffame is not None:
+            halloffame.update(offspring)
+
+        # Replace the current population by the offspring
+        population[:] = offspring
+
+        # Append the current generation statistics to the logbook
+        record = stats.compile(population) if stats else {}
+        logbook.record(gen=gen, nevals=len(invalid_ind), **record)
+        if verbose:
+            print logbook.stream
+
+    return population, logbook
+
+
 def graph(expr):
     """Construct the graph of a tree expression. The tree expression must be
     valid. It returns in order a node list, an edge list, and a dictionary of
@@ -947,6 +1202,129 @@ def graph(expr):
 
     return nodes, edges, labels
 
+
+######################################
+# GSGP Mutation                      #
+######################################
+
+def mutSemantic(individual, gen_func=genGrow, pset=None, ms=None, min=2, max=6):
+    """
+    Implementation of the Semantic Mutation operator. [Geometric semantic genetic programming, Moraglio et al., 2012]
+    mutated_individual = individual + logistic * (random_tree1 - random_tree2)
+
+    :param individual: individual to mutate
+    :param gen_func: function responsible for the generation of the random tree that will be used during the mutation
+    :param pset: Primitive Set, which contains terminal and operands to be used during the evolution
+    :param ms: Mutation Step
+    :param min: min depth of the random tree
+    :param max: max depth of the random tree
+    :return: mutated individual
+
+    The mutated contains the original individual
+
+        >>> import operator
+        >>> def lf(x): return 1 / (1 + math.exp(-x));
+        >>> pset = PrimitiveSet("main", 2)
+        >>> pset.addPrimitive(operator.sub, 2)
+        >>> pset.addTerminal(3)
+        >>> pset.addPrimitive(lf, 1, name="lf")
+        >>> pset.addPrimitive(operator.add, 2)
+        >>> pset.addPrimitive(operator.mul, 2)
+        >>> individual = genGrow(pset, 1, 3)
+        >>> mutated = mutSemantic(individual, pset=pset, max=2)
+        >>> ctr = sum([m.name == individual[i].name for i, m in enumerate(mutated[0])])
+        >>> ctr == len(individual)
+        True
+    """
+    for p in ['lf', 'mul', 'add', 'sub']:
+        assert p in pset.mapping, "A '" + p + "' function is required in order to perform semantic mutation"
+
+    tr1 = gen_func(pset, min, max)
+    tr2 = gen_func(pset, min, max)
+    # Wrap mutation with a logistic function
+    tr1.insert(0, pset.mapping['lf'])
+    tr2.insert(0, pset.mapping['lf'])
+    if ms is None:
+        ms = random.uniform(0, 2)
+    mutation_step = Terminal(ms, False, object)
+    # Create the root
+
+    new_ind = individual
+    new_ind.insert(0, pset.mapping["add"])
+    # Append the left branch
+    new_ind.append(pset.mapping["mul"])
+    new_ind.append(mutation_step)
+    new_ind.append(pset.mapping["sub"])
+    # Append the right branch
+    new_ind.extend(tr1)
+    new_ind.extend(tr2)
+
+    return new_ind,
+
+
+def cxSemantic(ind1, ind2, gen_func=genGrow, pset=None, min=2, max=6):
+    """
+    Implementation of the Semantic Crossover operator [Geometric semantic genetic programming, Moraglio et al., 2012]
+    offspring1 = random_tree1 * ind1 + (1 - random_tree1) * ind2
+    offspring2 = random_tree1 * ind2 + (1 - random_tree1) * ind1
+
+    :param ind1: first parent
+    :param ind2: second parent
+    :param gen_func: function responsible for the generation of the random tree that will be used during the mutation
+    :param pset: Primitive Set, which contains terminal and operands to be used during the evolution
+    :param min: min depth of the random tree
+    :param max: max depth of the random tree
+    :return: offsprings
+
+    The mutated offspring contains parents
+
+        >>> import operator
+        >>> def lf(x): return 1 / (1 + math.exp(-x));
+        >>> pset = PrimitiveSet("main", 2)
+        >>> pset.addPrimitive(operator.sub, 2)
+        >>> pset.addTerminal(3)
+        >>> pset.addPrimitive(lf, 1, name="lf")
+        >>> pset.addPrimitive(operator.add, 2)
+        >>> pset.addPrimitive(operator.mul, 2)
+        >>> ind1 = genGrow(pset, 1, 3)
+        >>> ind2 = genGrow(pset, 1, 3)
+        >>> new_ind1, new_ind2 = cxSemantic(ind1, ind2, pset=pset, max=2)
+        >>> ctr = sum([n.name == ind1[i].name for i, n in enumerate(new_ind1)])
+        >>> ctr == len(ind1)
+        True
+        >>> ctr = sum([n.name == ind2[i].name for i, n in enumerate(new_ind2)])
+        >>> ctr == len(ind2)
+        True
+    """
+    for p in ['lf', 'mul', 'add', 'sub']:
+        assert p in pset.mapping, "A '" + p + "' function is required in order to perform semantic crossover"
+
+    tr = gen_func(pset, min, max)
+    tr.insert(0, pset.mapping['lf'])
+    new_ind1 = ind1
+    new_ind1.insert(0, pset.mapping["mul"])
+    new_ind1.insert(0, pset.mapping["add"])
+    new_ind1.extend(tr)
+    new_ind1.append(pset.mapping["mul"])
+    new_ind1.append(pset.mapping["sub"])
+    new_ind1.append(Terminal(1.0, False, object))
+    new_ind1.extend(tr)
+    new_ind1.extend(ind2)
+
+    new_ind2 = ind2
+    new_ind2.insert(0, pset.mapping["mul"])
+    new_ind2.insert(0, pset.mapping["add"])
+    new_ind2.extend(tr)
+    new_ind2.append(pset.mapping["mul"])
+    new_ind2.append(pset.mapping["sub"])
+    new_ind2.append(Terminal(1.0, False, object))
+    new_ind2.extend(tr)
+    new_ind2.extend(ind1)
+
+    return new_ind1, new_ind2
+
+
 if __name__ == "__main__":
     import doctest
+
     doctest.testmod()
